@@ -232,6 +232,7 @@ class PillowPlugin(PluginV3):
         *,
         mode: str = None,
         format: str = None,
+        is_batch: bool = None,
         **kwargs,
     ) -> Optional[bytes]:
         """
@@ -247,17 +248,24 @@ class PillowPlugin(PluginV3):
 
         Parameters
         ----------
-        image : ndarray
-            The ndimage to write.
-        mode : {str, None}
+        image : ndarray or list
+            The ndimage to write. If a list is given each element is expected to
+            be an ndimage.
+        mode : str
             Specify the image's color format. If None (default), the mode is
             inferred from the array's shape and dtype. Possible modes can be
             found at:
             https://pillow.readthedocs.io/en/stable/handbook/concepts.html#modes
-        format : {str, None}
-            Optional format override.  If omitted, the format to use is
+        format : str
+            Optional format override. If omitted, the format to use is
             determined from the filename extension. If a file object was used
             instead of a filename, this parameter must always be used.
+        is_batch : bool
+            Explicitly tell the writer that ``image`` is a batch of images
+            (True) or not (False). If None, the writer will guess this from the
+            provided ``mode`` or ``image.shape``. While the latter often works,
+            it may cause problems for small images due to aliasing of spatial
+            and color-channel axes.
         kwargs : ...
             Extra arguments to pass to pillow. If a writer doesn't recognise an
             option, it is silently ignored. The available options are described
@@ -272,6 +280,11 @@ class PillowPlugin(PluginV3):
         image.
 
         """
+        if "fps" in kwargs:
+            raise TypeError(
+                "The keyword `fps` is no longer supported. Use `duration`"
+                "(in ms) instead, e.g. `fps=60` == `duration=1/60/1000`."
+            )
 
         extension = self.request.extension or self.request.format_hint
 
@@ -284,13 +297,12 @@ class PillowPlugin(PluginV3):
             is_batch = True
         else:
             ndimage = np.asarray(ndimage)
-            is_batch = None
 
         # check if ndimage is a batch of frames/pages (e.g. for writing GIF)
         # if mode is given, use it; otherwise fall back to image.ndim only
-        if is_batch is True:
-            pass  # ndimage was list; we know it is a batch
-        if mode is not None:
+        if is_batch is not None:
+            pass
+        elif mode is not None:
             is_batch = (
                 ndimage.ndim > 3 if Image.getmodebands(mode) > 1 else ndimage.ndim > 2
             )
